(function () {

  with (Monocle.Styles) {
    container.background = "none";
    container.right = "5px";
    container.left = '0';
    container.width = 'auto';
    page.top = page.bottom = "2px";
    page["-webkit-box-shadow"] = "1px 0 2px #997";
    page["-moz-box-shadow"] = "1px 0 2px #997";
    page["-webkit-border-top-left-radius"] = "26px 4px";
    page["-webkit-border-bottom-left-radius"] = "26px 4px";
    page["-moz-border-radius-topleft"] = "26px 4px";
    page["-moz-border-radius-bottomleft"] = "26px 4px";
    page['background-color'] = "#FFFEFC";
    page['background-image'] =
      "-moz-linear-gradient(0deg, #EDEAE8 0px, #FFFEFC 24px)";
    page.background =
      "-webkit-gradient(linear, 0 0, 24 0, from(#EDEAE8), to(#FFFEFC))";
    scroller.top = scroller.bottom = "2.5em";
    scroller.left = "1em";
    scroller.right = "1em";
    content.color = "#310";
    content["font-family"] = "Palatino, Georgia, serif";
    content["line-height"] = "130%";
    Controls.Magnifier.button.color = "#632";
    Controls.Magnifier.button.padding = "3px 6px";
    Controls.Magnifier.button['-webkit-border-radius'] = "3px";
    Controls.Magnifier.button.background = "#FFF";
    Controls.Magnifier.button.top = "8px";
    Controls.Contents.container.background = "#E0D3C0";
    Controls.Contents.container.border = "1px solid #EEd";
    Controls.Contents.list.font = "11pt Georgia, serif";
    Controls.Contents.list.color = "#642";
    Controls.Contents.list['text-shadow'] = "1px 1px #FFF6E0";
    Controls.Contents.chapter['border-bottom'] = "2px groove #FFF6E9";
  }

  var bookData = {
    getComponents: function () {
      var componentDiv = document.getElementById('components');
      var cmpts = [];
      for (var i = 0; i < componentDiv.childNodes.length; ++i) {
        var node = componentDiv.childNodes[i];
        if (node.nodeType == 1 && node.id) {
          cmpts.push(node.id);
        }
      }
      return cmpts;
    },
    getContents: function () {
      return [ $$TOCLIST$$ ]
    },
    getComponent: function (componentId) {
      return document.getElementById(componentId).innerHTML;
    },
    getMetaData: function (key) {
      return {
        title: "$$BOOKTITLE$$",
        creator: "$$BOOKCREATOR$$"
      }[key];
    }
  }


  function createToC(evt) {
    evt.preventDefault();
    evt.stopPropagation();
    var controlLayer = document.getElementById('readerCntr');

    tocList = document.createElement('ul');
    tocList.className = 'root';
    var listBuilder = function (chp, padLvl) {
      var li = document.createElement('li');
      var span = document.createElement('span');
      span.style.paddingLeft = padLvl + "em";
      li.appendChild(span);
      span.innerHTML = chp.title;
      li.onclick = function () {
        window.reader.skipToChapter(chp.src);
        controlLayer.removeChild(controlLayer.tocMenu);
      }
      tocList.appendChild(li);
      if (chp.children) {
        for (var i = 0; i < chp.children.length; ++i) {
          listBuilder(chp.children[i], padLvl + 1);
        }
      }
    }

    var contents = bookData.getContents();
    for (var i = 0; i < contents.length; ++i) {
      listBuilder(contents[i], 0);
    }

    if (!controlLayer.tocMenu) {
      controlLayer.tocMenu = document.createElement('div');
      controlLayer.tocMenu.id = "toc";
      controlLayer.tocMenu.appendChild(tocList);
      var arrow = document.createElement('div');
      arrow.className = "tocArrow";
      controlLayer.tocMenu.appendChild(arrow);
    }
    if (controlLayer.tocMenu.parentNode) {
      controlLayer.removeChild(controlLayer.tocMenu);
    } else {
      controlLayer.appendChild(controlLayer.tocMenu);
    }
  }


  // Initialize the reader element.
  Monocle.addListener(
    window,
    'load',
    function () {
      /* Initialize the reader */
      window.reader = Monocle.Reader('reader', bookData);

      /* Because the 'reader' element changes size on window resize,
       * we should notify it of this event. */
      Monocle.addListener(
        window,
        'resize',
        function () { window.reader.resized() }
      );


      /* SPINNER */
      var spinner = Monocle.Controls.Spinner(window.reader);
      window.reader.addControl(spinner, 'page', { hidden: true });
      spinner.listenForUsualDelays();


      /* PLACE SAVER */
      var placeSaver = new Monocle.Controls.PlaceSaver(reader);
      reader.addControl(placeSaver, 'invisible');
      var lastPlace = placeSaver.savedPlace();
      if (lastPlace) {
        placeSaver.restorePlace();
      }


      /* MAGNIFIER CONTROL */
      var magnifier = new Monocle.Controls.Magnifier(reader);
      reader.addControl(magnifier, 'page');


      /* CHAPTER TITLE RUNNING HEAD */
      var chapterTitle = {
        runners: [],
        createControlElements: function (page) {
          var cntr = document.createElement('div');
          cntr.className = "chapterTitle";
          var runner = document.createElement('div');
          runner.className = "runner";
          cntr.appendChild(runner);
          this.runners.push(runner);
          this.update(page);
          return cntr;
        },
        update: function (page) {
          var place = reader.getPlace(page);
          this.runners[page.pageIndex].innerHTML = place.chapterTitle();
        }
      }
      reader.addControl(chapterTitle, 'page');
      reader.addListener(
        'monocle:pagechange',
        function (evt) { chapterTitle.update(evt.monocleData.page); }
      );


      /* PAGE NUMBER RUNNING HEAD */
      var pageNumber = {
        runners: [],
        createControlElements: function (page) {
          var cntr = document.createElement('div');
          cntr.className = "pageNumber";
          var runner = document.createElement('div');
          runner.className = "runner";
          cntr.appendChild(runner);
          this.runners.push(runner);
          this.update(page);
          return cntr;
        },
        update: function (page) {
          var place = reader.getPlace(page);
          this.runners[page.pageIndex].innerHTML = place.pageNumber();
        }
      }
      reader.addControl(pageNumber, 'page');
      reader.addListener(
        'monocle:pagechange',
        function (evt) { pageNumber.update(evt.monocleData.page) }
      );


      /* Scrubber */
      var scrubber = new Monocle.Controls.Scrubber(reader);
      reader.addControl(scrubber, 'page', { hidden: true });
      var showFn = function (evt) {
        evt.stopPropagation();
        reader.showControl(scrubber);
        scrubber.updateNeedles();
      }
      var eType = "touchstart";
      for (var i = 0; i < chapterTitle.runners.length; ++i) {
        Monocle.addListener(chapterTitle.runners[i].parentNode, eType, showFn);
        Monocle.addListener(pageNumber.runners[i].parentNode, eType, showFn);
      }
      var hideScrubber = function (evt) {
        evt.stopPropagation();
        reader.hideControl(scrubber);
      }
      reader.addListener('monocle:contact:start', hideScrubber);
    }
  );
})();


function getPageInfo(){
	var place = reader.getPlace();
	var locus = place.getLocus();

	window.HybridApp.setPageIntent(locus.page, locus.componentId, place.chapterInfo().title);
}

function moveToBookMark(page, compId){
	var locus = {
      page: page,
      componentId: compId
    }

	reader.moveTo( locus );
}