Monocle = {
  VERSION: "1.0.0"
};

Monocle.pieceLoaded = function (piece) {
  if (typeof onMonoclePiece == 'function') {
    onMonoclePiece(piece);
  }
}

Monocle.Styles = {
  ruleText: function(rule) {
    if (typeof rule == "string") {
      rule = this[rule];
    }
    if (!rule) { return ""; }

    var parts = [];
    for (var declaration in rule) {
      parts.push(declaration + ": " + rule[declaration] + ";")
    }
    return parts.join(" ");
  }
}

Monocle.Flippers = {};
Monocle.Controls = {};

Monocle.pieceLoaded('monocle');

Monocle.addListener = function (elem, evtType, fn, useCapture) {
  if (elem.addEventListener) {
    return elem.addEventListener(evtType, fn, useCapture || false);
  } else if (elem.attachEvent) {
    return elem.attachEvent('on'+evtType, fn);
  }
}


Monocle.removeListener = function (elem, evtType, fn, useCapture) {
  if (elem.removeEventListener) {
    return elem.removeEventListener(evtType, fn, useCapture || false);
  } else if (elem.detachEvent) {
    try {
      return elem.detachEvent('on'+evtType, fn);
    } catch(e) {}
  }
}


if (typeof(MONOCLE_NO_COMPAT) == 'undefined') {

  if (typeof window.console == "undefined") {
    window.console = {
      messages: [],
      log: function (msg) {
        this.messages.push(msg);
      }
    }
  }


  if (!Array.prototype.indexOf) {
    Array.prototype.indexOf = function(elt /*, from*/) {
      var len = this.length >>> 0;

      var from = Number(arguments[1]) || 0;
      from = (from < 0)
        ? Math.ceil(from)
        : Math.floor(from);
      if (from < 0) {
        from += len;
      }

      for (; from < len; from++) {
        if (from in this && this[from] === elt) {
          return from;
        }
      }
      return -1;
    };
  }

}

Monocle.pieceLoaded('compat');

Monocle.Reader = function (node, bookData, options) {
  if (Monocle == this) { return new Monocle.Reader(node, bookData, options); }

  var k = {
    durations: {
      RESIZE_DELAY: 200
    },
    abortMessage: {
      CLASSNAME: "monocleAbortMessage",
      TEXT: "Your browser does not support this technology."
    },
    FLIPPER_DEFAULT_CLASS: (typeof(Monocle.Flippers.Slider) == "undefined") ?
      null :
      Monocle.Flippers.Slider,
    FLIPPER_LEGACY_CLASS: (typeof(Monocle.Flippers.Legacy) == "undefined") ?
      null :
      Monocle.Flippers.Legacy,
    TOUCH_DEVICE: true
  }

  var p = {
    divs: {
      box: null,
      container: null,
      overlay: null,
      pages: []
    },

    controls: [],

    pageWidth: 0,

    book: null,

    interactionData: {},

    resizeTimer: null
  }

  var API = {
    constructor: Monocle.Reader,
    properties: p,
    constants: k
  }


  function initialize(node, bookData, options) {
    p.divs.box = typeof(node) == "string" ?
      document.getElementById(node) :
      node;

    options = options || {}

    dispatchEvent("monocle:initializing");

    var bk;
    if (bookData) {
      bk = new Monocle.Book(bookData);
    } else {
      bk = Monocle.Book.fromHTML(p.divs.box.innerHTML);
    }
    p.divs.box.innerHTML = "";

    positionBox();

    attachFlipper(options.flipper);

    createReaderElements();

    dispatchEvent("monocle:loading");

    applyStyles();

    setBook(bk);

    listenForInteraction();

    dispatchEvent("monocle:loaded")
  }


  function positionBox() {
    var currPosVal;
    if (document.defaultView) {
      var currStyle = document.defaultView.getComputedStyle(p.divs.box, null);
      currPosVal = currStyle.getPropertyValue('position');
    } else if (p.divs.box.currentStyle) {
      currPosVal = p.divs.box.currentStyle.position
    }
    if (["absolute", "relative"].indexOf(currPosVal) == -1) {
      p.divs.box.style.position = "relative";
    }
  }


  function createReaderElements() {
    p.divs.container = document.createElement('div');
    p.divs.box.appendChild(p.divs.container);
    for (var i = 0; i < p.flipper.pageCount; ++i) {
      var page = p.divs.pages[i] = document.createElement('div');
      page.pageIndex = i;
      p.flipper.addPage(page);
      p.divs.container.appendChild(page);

      page.scrollerDiv = document.createElement('div');
      page.appendChild(page.scrollerDiv);

      page.contentDiv = document.createElement('div');
      page.scrollerDiv.appendChild(page.contentDiv);
    }
    p.divs.overlay = document.createElement('div');
    p.divs.box.appendChild(p.divs.overlay);
  }


  function attachFlipper(flipperClass) {
    if (navigator.product != "Gecko") { 
      if (!k.FLIPPER_LEGACY_CLASS) {
        var abortMsg = document.createElement('div');
        abortMsg.className = k.abortMessage.CLASSNAME;
        abortMsg.innerHTML = k.abortMessage.TEXT;
        p.divs.box.appendChild(abortMsg);
        return;
      }
      flipperClass = k.FLIPPER_LEGACY_CLASS;
    } else if (!flipperClass) {
      flipperClass = k.FLIPPER_DEFAULT_CLASS;
      if (!flipperClass) {
        throw("No flipper class");
      }
    }
    p.flipper = new flipperClass(API, setPage);
  }


  function applyStyles() {
    p.divs.container.style.cssText = Monocle.Styles.ruleText('container');
    for (var i = 0; i < p.flipper.pageCount; ++i) {
      var page = p.divs.pages[i];
      page.style.cssText = Monocle.Styles.ruleText('page');
      page.scrollerDiv.style.cssText = Monocle.Styles.ruleText('scroller');
      page.contentDiv.style.cssText = Monocle.Styles.ruleText('content');
    }
    p.divs.overlay.style.cssText = Monocle.Styles.ruleText('overlay');
  }


  function reapplyStyles() {
    applyStyles();
    calcDimensions();
  }


  function setBook(bk) {
    if (!dispatchEvent("monocle:bookchanging", {}, true)) {
      return;
    }
    p.book = bk;
    calcDimensions();
    dispatchEvent("monocle:bookchange");
    return p.book;
  }


  function getBook() {
    return p.book;
  }


  function resized() {
    if (!dispatchEvent("monocle:resizing", {}, true)) {
      return;
    }
    clearTimeout(p.resizeTimer);
    p.resizeTimer = setTimeout(
      function () {
        calcDimensions();
        dispatchEvent("monocle:resize");
      },
      k.durations.RESIZE_DELAY
    );
  }


  function calcDimensions() {
    p.boxDimensions = {
      left: 0,
      top: 0,
      width: p.divs.box.offsetWidth,
      height: p.divs.box.offsetHeight
    }
    var o = p.divs.box;
    do {
      p.boxDimensions.left += o.offsetLeft;
      p.boxDimensions.top += o.offsetTop;
    } while (o = o.offsetParent);

    if (typeof(p.flipper.overrideDimensions) != 'function') {
      var measuringPage = p.flipper.visiblePages()[0];
      p.pageWidth = measuringPage.offsetWidth;
      var cWidth = measuringPage.scrollerDiv.offsetWidth;
      for (var i = 0; i < p.divs.pages.length; ++i) {
        var cDiv = p.divs.pages[i].contentDiv;
        cDiv.style.webkitColumnWidth = cDiv.style.MozColumnWidth = cWidth+"px";
      }
    } else {
      p.flipper.overrideDimensions();
    }

    moveTo({ page: pageNumber() });
  }


  function pageNumber(pageDiv) {
    var place = p.flipper.getPlace(pageDiv);
    return place ? (place.pageNumber() || 1) : 1;
  }


  function getPlace(pageDiv) {
    return p.flipper.getPlace(pageDiv);
  }

  function moveTo(locus) {
    p.flipper.moveTo(locus);
  }


  function skipToChapter(src) {
    var page = p.flipper.visiblePages()[0];
    var place = p.book.placeOfChapter(page.contentDiv, src);
    moveTo(place.getLocus());
  }


  function setPage(pageDiv, locus, callback) {
    var eData = { page: pageDiv, locus: locus }

    if (!dispatchEvent('monocle:pagechanging', eData, true)) {
      return;
    }

    var rslt = p.book.changePage(pageDiv.contentDiv, locus);

    if (!rslt) {
      return false;
    }

    if (typeof callback == "function") {
      callback(rslt.offset);
    }

    eData.pageNumber = rslt.page;
    eData.componentId = rslt.componentId;
    dispatchEvent("monocle:pagechange", eData);

    return rslt.page;
  }


  function listenForInteraction() {
    if (!k.TOUCH_DEVICE) {
      Monocle.addListener(
        p.divs.container,
        'mousedown',
        function (evt) {
          if (evt.button != 0) {
            return;
          }
          p.interactionData.mouseDown = true;
          contactEvent(evt, "start", evt);
        }
      );
      Monocle.addListener(
        p.divs.container,
        'mousemove',
        function (evt) {
          if (!p.interactionData.mouseDown) {
            return false;
          }
          contactEvent(evt, "move", evt);
        }
      );
      Monocle.addListener(
        p.divs.container,
        'mouseup',
        function (evt) {
          if (!p.interactionData.mouseDown) {
            return false;
          }
          contactEvent(evt, "end", evt);
        }
      );
      Monocle.addListener(
        p.divs.container,
        'mouseout',
        function (evt) {
          if (!p.interactionData.mouseDown) {
            return false;
          }
          obj = evt.relatedTarget || e.fromElement;
          while (obj && (obj = obj.parentNode)) {
            if (obj == p.divs.container) { return; }
          }
          contactEvent(evt, 'end', evt);
        }
      );
    } else {
      Monocle.addListener(
        p.divs.container,
        'touchstart',
        function (evt) {
          if (evt.touches.length > 1) { return; }
          contactEvent(evt, 'start', evt.targetTouches[0]);
          evt.preventDefault();
        }
      );
      Monocle.addListener(
        p.divs.container,
        'touchmove',
        function (evt) {
          if (evt.touches.length > 1) { return; }
          var raw = {
            x: evt.targetTouches[0].pageX - p.boxDimensions.left,
            y: evt.targetTouches[0].pageY - p.boxDimensions.top,
            w: p.boxDimensions.width,
            h: p.boxDimensions.height
          }
          if (raw.x < 0 || raw.y < 0 || raw.x >= raw.w || raw.y >= raw.h) {
            contactEvent(evt, "end", evt.targetTouches[0]);
          } else {
            contactEvent(evt, "move", evt.targetTouches[0]);
          }
          evt.preventDefault();
        }
      );
      Monocle.addListener(
        p.divs.container,
        'touchend',
        function (evt) {
          contactEvent(evt, "end", evt.changedTouches[0]);
          evt.preventDefault();
        }
      );
      Monocle.addListener(
        p.divs.container,
        'touchcancel',
        function (evt) {
          contactEvent(evt, "end", evt.changedTouches[0]);
        }
      );
      Monocle.addListener(window, 'orientationchange', resized, true);
    }
    p.flipper.listenForInteraction();
  }


  function contactEvent(evt, eType, cursorInfo) {
    cData = {
      contactX: Math.min(
        p.boxDimensions.width,
        Math.max(0, cursorInfo.pageX - p.boxDimensions.left)
      ),
      contactY: Math.min(
        p.boxDimensions.height,
        Math.max(0, cursorInfo.pageY - p.boxDimensions.top)
      )
    };
    if (
      !dispatchEvent("monocle:contact:"+eType, cData, true) ||
      !dispatchEvent("monocle:contact:"+eType+":unhandled", cData, true)
    ) {
      evt.preventDefault();
    }

    if (eType == "end") {
      p.interactionData = {};
    }
  }


  function addControl(ctrl, cType, options) {
    for (var i = 0; i < p.controls.length; ++i) {
      if (p.controls[i].control == ctrl) {
        console.log("Already added control: " + ctrl);
        return;
      }
    }

    options = options || {};

    var ctrlData = {
      control: ctrl,
      elements: [],
      controlType: cType
    }
    p.controls.push(ctrlData);

    var ctrlElem;
    if (!cType || cType == "standard") {
      ctrlElem = ctrl.createControlElements(p.divs.container);
      p.divs.container.appendChild(ctrlElem);
      ctrlData.elements.push(ctrlElem);
    } else if (cType == "page") {
      for (var i = 0; i < p.divs.pages.length; ++i) {
        var page = p.divs.pages[i];
        var runner = ctrl.createControlElements(page);
        page.appendChild(runner);
        ctrlData.elements.push(runner);
      }
    } else if (cType == "modal" || cType == "popover") {
      ctrlElem = ctrl.createControlElements(p.divs.overlay);
      p.divs.overlay.appendChild(ctrlElem);
      ctrlData.elements.push(ctrlElem);
      ctrlData.usesOverlay = true;
    } else if (cType == "invisible") {
      if (
        typeof(ctrl.createControlElements) == "function" &&
        (ctrlElem = ctrl.createControlElements(p.divs.container))
      ) {
          p.divs.container.appendChild(ctrlElem);
          ctrlData.elements.push(ctrlElem);
      }
    } else {
      console.log("Unknown control type: " + cType);
    }

    for (var i = 0; i < ctrlData.elements.length; ++i) {
      ctrlData.elements[i].style.cssText += Monocle.Styles.ruleText('control');
    }

    if (options.hidden) {
      hideControl(ctrl);
    } else {
      showControl(ctrl);
    }

    return ctrl;
  }


  function dataForControl(ctrl) {
    for (var i = 0; i < p.controls.length; ++i) {
      if (p.controls[i].control == ctrl) {
        return p.controls[i];
      }
    }
  }


  function hideControl(ctrl) {
    var controlData = dataForControl(ctrl);
    if (!controlData) {
      throw("No data for control: " + ctrl);
    }
    if (controlData.hidden) {
      return;
    }
    for (var i = 0; i < controlData.elements.length; ++i) {
      controlData.elements[i].style.display = "none";
    }
    if (controlData.usesOverlay) {
      p.divs.overlay.style.display = "none";
      var evtType = k.TOUCH_DEVICE ? "touchstart" : "mousedown";
      Monocle.removeListener(p.divs.overlay, evtType, p.divs.overlay.clickFn);
    }
    controlData.hidden = true;
    if (ctrl.properties) {
      ctrl.properties.hidden = true;
    }
  }


  function showControl(ctrl) {
    var controlData = dataForControl(ctrl);
    if (!controlData) {
      throw("No data for control: " + ctrl);
    }
    if (controlData.hidden == false) {
      return;
    }
    for (var i = 0; i < controlData.elements.length; ++i) {
      controlData.elements[i].style.display = "block";
    }
    if (controlData.usesOverlay) {
      p.divs.overlay.style.display = "block";
    }
    if (controlData.controlType == "popover") {
      p.divs.overlay.clickFn = function (evt) {
        obj = evt.target || window.event.srcElement;
        do {
          if (obj == controlData.elements[0]) { return true; }
        } while (obj && (obj = obj.parentNode));
        hideControl(ctrl);
      }
      var evtType = k.TOUCH_DEVICE ? "touchstart" : "mousedown";
      Monocle.addListener(p.divs.overlay, evtType, p.divs.overlay.clickFn);
    }
    controlData.hidden = false;
    if (ctrl.properties) {
      ctrl.properties.hidden = false;
    }
  }


  function dispatchEvent(evtType, data, cancelable) {
    if (!document.createEvent) {
      return true;
    }
    var evt = document.createEvent("Events");
    evt.initEvent(evtType, false, cancelable || false);
    evt.monocleData = data;
    return p.divs.box.dispatchEvent(evt);
  }


  function addListener(evtType, fn, useCapture) {
    Monocle.addListener(p.divs.box, evtType, fn, useCapture);
  }


  API.setBook = setBook;
  API.getBook = getBook;
  API.reapplyStyles = reapplyStyles;
  API.getPlace = getPlace;
  API.moveTo = moveTo;
  API.skipToChapter = skipToChapter;
  API.resized = resized;
  API.addControl = addControl;
  API.hideControl = hideControl;
  API.showControl = showControl;
  API.dispatchEvent = dispatchEvent;
  API.addListener = addListener;

  initialize(node, bookData, options);

  return API;
}

Monocle.pieceLoaded('reader');

Monocle.Book = function (dataSource) {
  if (Monocle == this) { return new Monocle.Book(dataSource); }

  var k = {
  }

  var p = {
    dataSource: dataSource,
    components: [],
    places: [],
    chapters: {} 
  }

  var API = {
    constructor: Monocle.Book,
    constants: k,
    properties: p
  }


  function initialize() {
    p.componentIds = dataSource.getComponents();
    p.contents = dataSource.getContents();
    p.lastCIndex = p.componentIds.length - 1;
  }


  function changePage(node, locus) {
    var place = placeFor(node) || setPlaceFor(node, componentAt(0), 1);

    var cIndex = p.componentIds.indexOf(locus.componentId);
    var component;
    if (cIndex == -1) {
      component = place.properties.component;
    } else {
      component = componentAt(cIndex);
    }

    if (component != place.properties.component) {
      component.applyTo(node);
    }

    var oldCmptLPN = component.lastPageNumber();
    var changedDims = component.updateDimensions(node);

    var pageN = 1;
    if (typeof(locus.page) == "number") {
      pageN = locus.page;
    } else if (typeof(locus.percent) == "number") {
      place = setPlaceFor(node, component, 1);
      pageN = place.pageAtPercentageThrough(locus.percent);
    } else if (typeof(locus.direction) == "number") {
      pageN = place.pageNumber();
      pageN += locus.direction;
    } else if (typeof(locus.anchor) == "string") {
      if (locus.position == "start") {
        pageN = 1;
      } else if (locus.position == "end") {
        pageN = component.lastPageNumber();
      }
    } else {
      console.log("Unrecognised locus: " + locus);
    }

    if (changedDims && parseInt(oldCmptLPN)) {
      pageN = Math.max(
        Math.round(component.lastPageNumber() * (pageN / oldCmptLPN)),
        1
      );
    }

    cIndex = component.properties.index;
    var lpn = component.lastPageNumber();
    if (cIndex == 0 && pageN < 1) {
      return false;
    } else if (cIndex == p.lastCIndex && pageN > component.lastPageNumber()) {
      return false;
    } else if (pageN > component.lastPageNumber()) {
      pageN -= component.lastPageNumber();
      component = componentAt(cIndex + 1);
      return changePage(
        node,
        { page: pageN, componentId: component.properties.id }
      );
    } else if (pageN < 1) {
      component = componentAt(cIndex - 1);
      component.updateDimensions(node);
      pageN += component.lastPageNumber();
      return changePage(
        node,
        { page: pageN, componentId: component.properties.id }
      );
    }

    component.prepareNode(node, pageN)
    setPlaceFor(node, component, pageN);

    return {
      componentId: component.properties.id,
      page: pageN,
      offset: (pageN - 1) * node.parentNode.offsetWidth
    }
  }


  function placeFor(node) {
    for (var i = p.places.length - 1; i >= 0; --i) {
      if (p.places[i][0] == node) {
        return p.places[i][1];
      }
    }
    return null;
  }


  function setPlaceFor(node, component, pageN) {
    var place = placeFor(node);
    if (!place) {
      place = new Monocle.Place();
      p.places[p.places.length] = [node, place];
    }
    place.setPlace(component, pageN);
    return place;
  }


  function componentAt(index) {
    if (!p.components[index]) {
      var src = p.componentIds[index];
      var html = p.dataSource.getComponent(src);
      p.components[index] = new Monocle.Component(
        API,
        src,
        index,
        chaptersForComponent(src),
        html
      );
    }
    return p.components[index];
  }


  function chaptersForComponent(src) {
    if (p.chapters[src]) {
      return p.chapters[src];
    }
    p.chapters[src] = [];
    var matcher = new RegExp('^'+src+"(\#(.+)|$)");
    var matches;
    var recurser = function (chp) {
      if (matches = chp.src.match(matcher)) {
        p.chapters[src].push({
          title: chp.title,
          fragment: matches[2] || null
        });
      }
      if (chp.children) {
        for (var i = 0; i < chp.children.length; ++i) {
          recurser(chp.children[i]);
        }
      }
    }

    for (var i = 0; i < p.contents.length; ++i) {
      recurser(p.contents[i]);
    }
    return p.chapters[src];
  }


  function placeOfChapter(node, src) {
    var matcher = new RegExp('^(.+?)(#(.*))?$');
    var matches = src.match(matcher);
    if (matches) {
      var cmptId = matches[1];
      var fragment = matches[3] || null;
      var cIndex = p.componentIds.indexOf(cmptId);
      var component = componentAt(cIndex);
      component.updateDimensions(node);
      var place = new Monocle.Place(node);
      if (fragment) {
        place.setPlace(component, component.pageForChapter(fragment));
      } else {
        place.setPlace(component, 1);
      }
      return place;
    }
    return null;
  }


  function chapterTree() {
    return p.contents;
  }


  API.getMetaData = dataSource.getMetaData;
  API.changePage = changePage;
  API.chapterTree = chapterTree;
  API.chaptersForComponent = chaptersForComponent;
  API.placeFor = placeFor;
  API.placeOfChapter = placeOfChapter;

  initialize();

  return API;
}


Monocle.Book.fromHTML = function (html) {
  var bookData = {
    getComponents: function () {
      return ['anonymous'];
    },
    getContents: function () {
      return [];
    },
    getComponent: function (n) {
      return html;
    },
    getMetaData: function (key) {
    }
  }

  return new Monocle.Book(bookData);
}

Monocle.pieceLoaded('book');
Monocle.Component = function (book, id, index, chapters, html) {
  if (Monocle == this) {
    return new Monocle.Component(book, id, index, chapters, html);
  }

  var k = {
  }

  var p = {
    book: book,
    id: id,
    index: index,
    chapters: chapters,
    html: html,
    clientNodes: [],
    elementsForClient: [],
    chunks: [],
    clientDimensions: []
  }

  var API = {
    constructor: Monocle.Component,
    constants: k,
    properties: p
  }


  function initialize() {
    if (!p.html) {
      console.log("Accessed an empty component: " + p.id);
      p.html = "<p></p>"
    }

    var elems = p.elementsForClient[0] = [];

    var tmpDiv = document.createElement('div');
    tmpDiv.innerHTML = p.html;
    while (tmpDiv.hasChildNodes()) {
      var node = tmpDiv.removeChild(tmpDiv.firstChild);
      if (node.nodeType == 1) {
        elems.push(node);
      } else if (node.nodeType == 3 && !node.nodeValue.match(/^\s+$/)) {
        var elem = document.createElement('div');
        elem.appendChild(node)
        elems.push(elem);
      }
    }
    delete(tmpDiv);
  }


  function nodeIndex(node) {
    return p.clientNodes.indexOf(node);
  }


  function prepareNode(node, pageN) {
    for (var i = 0; i < p.chunks.length; ++i) {
      if (p.chunks[i].firstPageNumber - 1 <= pageN) {
        appendChunk(node, p.chunks[i]);
      } else {
        detachChunk(node, p.chunks[i]);
      }
    }
  }


  function chapterForPage(pageN) {
    var cand = null;
    for (var i = 0; i < p.chapters.length; ++i) {
      if (pageN >= p.chapters[i].page) {
        cand = p.chapters[i];
      } else {
        return cand;
      }
    }
    return cand;
  }


  function pageForChapter(fragment) {
    if (!fragment) {
      return 1;
    }
    for (var i = 0; i < p.chapters.length; ++i) {
      if (p.chapters[i].fragment == fragment) {
        return p.chapters[i].page;
      }
    }
    return null;
  }


  function appendChunk(node, chunk) {
    var slice = p.elementsForClient[nodeIndex(node)].slice(
      chunk.firstElementIndex,
      chunk.lastElementIndex
    );

    if (slice[0].parentNode == node) {
      return;
    }

    addElementsTo(node, slice);
  }


  function detachChunk(node, chunk) {
    var slice = p.elementsForClient[nodeIndex(node)].slice(
      chunk.firstElementIndex,
      chunk.lastElementIndex
    );

    if (slice[0].parentNode != node) {
      return;
    }

    removeElementsFrom(node, slice);
  }


  function applyTo(node) {
    registerClient(node);
    removeElementsFrom(node);
  }


  function updateDimensions(node) {
    registerClient(node);

    if (haveDimensionsChanged(node)) {
      removeElementsFrom(node);
      addElementsTo(node, p.elementsForClient[nodeIndex(node)]);
      measureDimensions(node);
      locateChapters(node);
      primeChunks(node);

      for (var i = 0; i < p.clientNodes.length; ++i) {
        removeElementsFrom(p.clientNodes[i]);
      }

      return true;
    } else {
      return false;
    }
  }


  function registerClient(node) {
    if (nodeIndex(node) != -1) {
      return;
    }
    p.clientNodes.push(node);

    if (!p.elementsForClient[nodeIndex(node)]) {
      var sourceElems = p.elementsForClient[0];
      var destElems = p.elementsForClient[nodeIndex(node)] = [];
      var len = sourceElems.length;
      for (var i = 0; i < len; ++i) {
        destElems[i] = sourceElems[i].cloneNode(true);
      }
    }
  }


  function haveDimensionsChanged(node) {
    return (!p.clientDimensions) ||
      (p.clientDimensions.width != node.parentNode.offsetWidth) ||
      (p.clientDimensions.height != node.parentNode.offsetHeight) ||
      (p.clientDimensions.fontSize != node.style.fontSize);
  }


  function positionImages(node) {
    if (!node.getBoundingClientRect) {
      console.log('Image positioning not supported');
      return;
    } else {
      console.log('Positioning images to top of pages');
    }
    var cRect = node.getBoundingClientRect();
    var imgs = node.getElementsByTagName('img');
    for (var i = 0; i < imgs.length; ++i) {
      var iRect = imgs[i].getBoundingClientRect();
      if (iRect.top == cRect.top) {
        imgs[i].style.marginTop = 0;
      } else {
        imgs[i].style.marginTop = (cRect.height - (iRect.top - cRect.top))+"px";
      }
    }
  }


  function measureDimensions(node) {
    p.clientDimensions = {
      width: node.parentNode.offsetWidth,
      height: node.parentNode.offsetHeight,
      scrollWidth: node.parentNode.scrollWidth,
      fontSize: node.style.fontSize
    }

    if (p.clientDimensions.scrollWidth == p.clientDimensions.width * 2) {
      var lcEnd = node.lastChild.offsetTop + node.lastChild.offsetHeight;
      p.clientDimensions.scrollWidth = p.clientDimensions.width *
        (lcEnd > p.clientDimensions.height ? 2 : 1);
    }

    p.clientDimensions.pages = Math.ceil(
      p.clientDimensions.scrollWidth / p.clientDimensions.width
    );

    return p.clientDimensions;
  }


  function locateChapters(node) {
    for (var i = 0; i < p.chapters.length; ++i) {
      var chp = p.chapters[i];
      chp.page = 1;
      if (chp.fragment) {
        var target = document.getElementById(chp.fragment);
        while (target && target.parentNode != node) {
          target = target.parentNode;
        }
        if (target) {
          target.scrollIntoView();
          chp.page = (node.parentNode.scrollLeft / p.clientDimensions.width) + 1;
        }
      }
    }
    node.parentNode.scrollLeft = 0;

    return p.chapters;
  }


  function tmpLocateOcclusions(node) {
    if (!node.getBoundingClientRect) {
      console.log('Occlusion not supported');
      return;
    } else {
      console.log('Locating occlusions');
    }

    var topElems = [];
    var cRect = node.getBoundingClientRect();
    for (var i = 0; i < node.childNodes.length; ++i) {
      var elem = node.childNodes[i];
      var prevElem = node.childNodes[i - 1];
      if (!prevElem) {
        topElems.push(elem);
      } else {
        var nRect = elem.getBoundingClientRect();
        var pRect = prevElem.getBoundingClientRect();
        if (pRect.bottom <= cRect.bottom && nRect.top <= pRect.top) {
          topElems.push(elem);
        }
      }
      elem.style.color = "#000";
    }

    for (i = 0; i < topElems.length; ++i) {
      topElems[i].style.color = "#F0F"; 
    }
  }


  function primeChunks(node) {
    p.chunks = [];
    var elements = p.elementsForClient[nodeIndex(node)];
    var pagesRemaining = p.clientDimensions.pages;
    var chunkSize = Math.ceil(elements.length / (pagesRemaining / 4));
    var chunkCount = Math.ceil(elements.length / chunkSize);

    var elemCount = 0;
    for (var i = 0; i < chunkCount; ++i) {
      for (var j = 0; j < chunkSize && node.hasChildNodes(); ++j, ++elemCount) {
        node.removeChild(node.firstChild);
      }
      var newPagesRemaining = Math.floor(
        node.parentNode.scrollWidth / p.clientDimensions.width
      );
      p.chunks.push({
        firstElementIndex: elemCount - j,
        lastElementIndex: elemCount,
        firstPageNumber: (p.clientDimensions.pages - pagesRemaining) + 1,
        lastPageNumber: (p.clientDimensions.pages - newPagesRemaining) + 1
      });
      pagesRemaining = newPagesRemaining;
    }

    return p.chunks;
  }


  function addElementsTo(node, elementArray) {
    var len = elementArray.length;
    for (var i = 0; i < len; ++i) {
      node.appendChild(elementArray[i]);
    }
    return len;
  }


  function removeElementsFrom(node, elementArray) {
    var len;
    if (elementArray) {
      len = elementArray.length;
      for (var i = 0; i < len; ++i) {
        if (elementArray[i].parentNode == node) {
          node.removeChild(elementArray[i]);
        }
      }
      return len;
    }

    len = node.childNodes.length;
    while (node.hasChildNodes()) {
      node.removeChild(node.firstChild);
    }
    return len;
  }

  function lastPageNumber() {
    return p.clientDimensions ? p.clientDimensions.pages : null;
  }


  API.applyTo = applyTo;
  API.updateDimensions = updateDimensions;
  API.lastPageNumber = lastPageNumber;
  API.prepareNode = prepareNode;
  API.chapterForPage = chapterForPage;
  API.pageForChapter = pageForChapter;

  initialize();

  return API;
}

Monocle.pieceLoaded('component');

Monocle.Place = function () {
  if (Monocle == this) { return new Monocle.Place(); }

  var k = {
  }

  var p = {
    component: null,
    percent: null
  }

  var API = {
    constructor: Monocle.Place,
    constants: k,
    properties: p
  }


  function setPlace(cmpt, pageN) {
    p.component = cmpt;
    p.percent = pageN / cmpt.lastPageNumber();
    p.chapter = null;
  }


  function setPercentageThrough(cmpt, percent) {
    p.component = cmpt;
    p.percent = percent;
    p.chapter = null;
  }


  function componentId() {
    return p.component.properties.id;
  }


  function percentageThrough() {
    return p.percent;
  }


  function pageAtPercentageThrough(pc) {
    return Math.round(p.component.lastPageNumber() * pc);
  }


  function pageNumber() {
    return pageAtPercentageThrough(p.percent);
  }


  function chapterInfo() {
    if (p.chapter) {
      return p.chapter;
    }
    return p.chapter = p.component.chapterForPage(pageNumber());
  }


  function chapterTitle() {
    var chp = chapterInfo();
    return chp ? chp.title : null;
  }


  function getLocus(options) {
    options = options || {};
    var locus = {
      page: pageNumber(),
      componentId: componentId()
    }
    if (options.direction) {
      locus.page += options.direction;
    }
    return locus;
  }


  API.setPlace = setPlace;
  API.setPercentageThrough = setPercentageThrough;
  API.componentId = componentId;
  API.percentageThrough = percentageThrough;
  API.pageAtPercentageThrough = pageAtPercentageThrough;
  API.pageNumber = pageNumber;
  API.chapterInfo = chapterInfo;
  API.chapterTitle = chapterTitle;
  API.getLocus = getLocus;

  return API;
}


Monocle.Place.FromPageNumber = function (component, pageNumber) {
  var place = new Monocle.Place();
  place.setPlace(component, pageNumber);
  return place;
}

Monocle.Place.FromPercentageThrough = function (component, percent) {
  var place = new Monocle.Place();
  place.setPercentageThrough(component, percent);
  return place;
}

Monocle.pieceLoaded('place');
Monocle.Styles.container = {
  "position": "absolute",
  "width": "100%",
  "height": "100%",
  "-webkit-user-select": "none",
  "-webkit-text-size-adjust": "none"
}

Monocle.Styles.overlay = {
  "position": "absolute",
  "display": "none",
  "width": "100%",
  "height": "100%",
  "z-index": "1000"
}

Monocle.Styles.page = {
  "position": "absolute",
  "top": "0",
  "left": "0",
  "bottom": "3px",
  "right": "5px",
  "background": "#FFF",
  "cursor": "pointer",
  "z-index": "1",
  "-webkit-box-shadow": "2px 0 2px #999",
  "-moz-box-shadow": "2px 0 2px #999",
  "-webkit-transform-style": "preserve-3d"
}

Monocle.Styles.scroller = {
  "position": "absolute",
  "top": "1em",
  "bottom": "1em",
  "left": "1em",
  "right": "1em",
  "overflow": "hidden"
}

Monocle.Styles.content = {
  "position": "absolute",
  "top": "0",
  "bottom": "0",
  "min-width": "200%",
  "-webkit-text-size-adjust": "none",
  "-webkit-column-gap": "0",
  "-webkit-column-fill": "auto",
  "-moz-column-gap": "0",
  "-moz-column-fill": "auto"
}

Monocle.Styles.spinner = {
  "width": "48px",
  "height": "48px",
  "position": "relative",
  "display": "block",
  "margin": "auto"
}

Monocle.Styles.control = {
  "z-index": "100" 
}

Monocle.Styles.Controls = {
}

Monocle.Styles.Flippers = {
}

Monocle.pieceLoaded('styles');
Monocle.Flippers.Slider = function (reader, setPageFn) {
  if (Monocle.Flippers == this) {
    return new Monocle.Flippers.Slider(reader, setPageFn);
  }

  var k = {
    FORWARDS: 1,
    BACKWARDS: -1,
    durations: {
      SLIDE: 200,
      FOLLOW_CURSOR: 100,
      ANTI_FLICKER_DELAY: 20
    }
  }


  var p = {
    pageCount: 2,
    activeIndex: 1,
    divs: {
      pages: []
    },
    turnData: {}
  }

  var API = {
    constructor: Monocle.Flippers.Slider,
    properties: p,
    constants: k
  }


  function initialize() {
    p.reader = reader;
    p.setPageFn = setPageFn;
  }


  function addPage(pageDiv) {
    p.divs.pages.push(pageDiv);
  }


  function visiblePages() {
    return [upperPage()];
  }


  function listenForInteraction() {
    p.reader.addListener(
      "monocle:contact:start",
      function (evt) {
        if (lift(evt.monocleData.contactX)) {
          evt.preventDefault();
        }
      }
    );
    p.reader.addListener(
      "monocle:contact:move",
      function (evt) {
        if (turning(evt.monocleData.contactX)) {
          evt.preventDefault();
        }
      }
    );
    p.reader.addListener(
      "monocle:contact:end",
      function (evt) {
        if (drop(evt.monocleData.contactX)) {
          evt.preventDefault();
        }
      }
    );
  }


  function getPlace(pageDiv) {
    pageDiv = pageDiv || upperPage();
    return p.reader.getBook().placeFor(pageDiv.contentDiv);
  }


  function moveTo(locus) {
    setPage(upperPage(), locus);
    completedTurn();
  }


  function setPage(pageDiv, locus, callback) {
    var spCallback = function (offset) {
      pageDiv.scrollerDiv.scrollLeft = offset;
      setX(pageDiv.scrollerDiv, 0, { duration: 0 }, callback);
    }
    return p.setPageFn(pageDiv, locus, spCallback);
  }


  function inForwardZone(x) {
    return x > p.reader.properties.pageWidth * 0.6;
  }


  function inBackwardZone(x) {
    return x < p.reader.properties.pageWidth * 0.4;
  }


  function upperPage() {
    return p.divs.pages[p.activeIndex];
  }


  function lowerPage() {
    return p.divs.pages[(p.activeIndex + 1) % 2];
  }


  function flipPages() {
    upperPage().style.zIndex = 1;
    lowerPage().style.zIndex = 2;
    return p.activeIndex = (p.activeIndex + 1) % 2;
  }


  function deferredCall(fn) {
    setTimeout(fn, k.durations.ANTI_FLICKER_DELAY);
  }


  function liftAnimationFinished(boxPointX) {
    p.turnData.animating = false;
    if (p.turnData.dropped) {
      drop(boxPointX);
      p.turnData.dropped -= 1;
    }
  }


  function onLastPage() {
    var upperPlace = getPlace(upperPage());
    var lowerPlace = getPlace(lowerPage());
    return (
      upperPlace.componentId() == lowerPlace.componentId() &&
      upperPlace.pageNumber() == lowerPlace.pageNumber()
    )
  }


  function lift(boxPointX) {
    if (p.turnData.animating || p.turnData.direction) {
      return false;
    }

    p.turnData.points = {
      start: boxPointX,
      min: boxPointX,
      max: boxPointX
    }

    if (inForwardZone(boxPointX)) {
      if (!onLastPage()) {
        p.turnData.direction = k.FORWARDS;
        slideToCursor(boxPointX);
        liftAnimationFinished();
      }
      return true;
    } else if (inBackwardZone(boxPointX)) {
      p.turnData.animating = true;
      var place = getPlace();
      var pageSetSuccessfully = setPage(
        lowerPage(),
        place.getLocus({ direction: k.BACKWARDS }),
        function () {
          p.turnData.direction = k.BACKWARDS;
          deferredCall(function() {
            jumpOut(function () {
              deferredCall(function () {
                flipPages();
                slideToCursor(boxPointX);
                liftAnimationFinished(boxPointX);
              });
            });
          });
        }
      );

      if (!pageSetSuccessfully) {
        p.turnData = {};
        return false;
      }
      return true;
    }
    return false;
  }


  function turning(boxPointX) {
    if (p.turnData.animating || !p.turnData.direction) {
      return false;
    }

    p.turnData.points.min = Math.min(p.turnData.points.min, boxPointX);
    p.turnData.points.max = Math.max(p.turnData.points.max, boxPointX);

    var stamp = (new Date()).getTime();
    var followInterval = k.durations.FOLLOW_CURSOR;
    if (
      p.turnData.stamp &&
      stamp - p.turnData.stamp < followInterval
    ) {
      return false;
    }
    p.turnData.stamp = stamp;

    slideToCursor(boxPointX);

    return true;
  }


  function drop(boxPointX) {
    if (p.turnData.animating) {
      p.turnData.dropped = true;
      return true;
    }
    if (!p.turnData.direction) {
      return false;
    }

    p.turnData.animating = true;

    p.turnData.points.tap = p.turnData.points.max - p.turnData.points.min < 10;

    if (p.turnData.direction == k.FORWARDS) {
      if (
        p.turnData.points.tap ||
        p.turnData.points.start - boxPointX > 60 ||
        p.turnData.points.min >= boxPointX
      ) {
        slideOut(flipPages);
      } else {
        slideIn();
      }
      return true;
    } else if (p.turnData.direction == k.BACKWARDS) {
      if (
        p.turnData.points.tap ||
        boxPointX - p.turnData.points.start > 60 ||
        p.turnData.points.max <= boxPointX
      ) {
        slideIn();
      } else {
        slideOut(flipPages);
      }
      return true;
    }
    return false;
  }


  function completedTurn() {
    var place = getPlace();
    var resetTurn = function () {
      p.reader.dispatchEvent('monocle:turn');
      p.turnData = {};
    }

    if (
      !setPage(
        lowerPage(),
        place.getLocus({ direction: k.FORWARDS }),
        function () {
          jumpIn(resetTurn);
        }
      )
    ) {
      setPage(
        lowerPage(),
        place.getLocus(),
        function () {
          jumpIn(resetTurn);
        }
      );
    }
  }


  function setX(elem, x, options, callback) {
    var transition;
    var duration;

    if (typeof(x) == "number") { x = x + "px"; }

    if (!options.duration) {
      duration = 0;
      transition = 'none';
    } else {
      duration = parseInt(options['duration']);
      transition = '-webkit-transform';
      transition += ' ' + duration + "ms";
      transition += ' ' + (options['timing'] || 'linear');
      transition += ' ' + (options['delay'] || 0) + 'ms';
    }

    if (typeof WebKitTransitionEvent != "undefined") {
      elem.style.webkitTransition = transition;
      elem.style.webkitTransform = "translateX("+x+")";
    } else if (transition != "none") {
      clearTimeout(elem.setXTransitionInterval)

      var stamp = (new Date()).getTime();
      var frameRate = 40;
      var finalX = parseInt(x);
      var currX = getX(elem);
      var step = (finalX - currX) * (frameRate / duration);
      var stepFn = function () {
        var destX = currX + step;
        if (
          (new Date()).getTime() - stamp > duration ||
          Math.abs(currX - finalX) <= Math.abs((currX + step) - finalX)
        ) {
          clearTimeout(elem.setXTransitionInterval)
          elem.style.MozTransform = "translateX(" + finalX + "px)";
          if (elem.setXTCB) {
            elem.setXTCB();
          }
        } else {
          elem.style.MozTransform = "translateX(" + destX + "px)";
          currX = destX;
        }
      }

      elem.setXTransitionInterval = setInterval(stepFn, frameRate);
    } else {
      elem.style.MozTransform = "translateX("+x+")";
    }

    if (elem.setXTCB) {
      Monocle.removeListener(elem, 'webkitTransitionEnd', elem.setXTCB);
      elem.setXTCB = null;
    }
    if (callback) {
      if (transition == "none" || getX(elem) == parseInt(x)) {
        callback();
      } else {
        elem.setXTCB = callback;
        Monocle.addListener(elem, 'webkitTransitionEnd', elem.setXTCB);
      }
    }
  }


  function getX(elem) {
    if (typeof WebKitCSSMatrix == "object") {
      var matrix = window.getComputedStyle(elem).webkitTransform;
      matrix = new WebKitCSSMatrix(matrix);
      return matrix.m41;
    } else {
      var prop = elem.style.MozTransform;
      if (!prop || prop == "") { return 0; }
      return parseFloat((/translateX\((\-?.*)px\)/).exec(prop)[1]) || 0;
    }
  }



  function jumpIn(callback) {
    setX(lowerPage(), 0, { duration: 1 }, callback);
  }


  function jumpOut(callback) {
    setX(
      lowerPage(),
      0 - p.reader.properties.pageWidth,
      { duration: 0 },
      callback
    );
  }


  function jumpToCursor(cursorX, callback) {
    setX(
      lowerPage(),
      Math.min(0, cursorX - p.reader.properties.pageWidth),
      { duration: 0 },
      callback
    );
  }


  function slideIn(callback) {
    var slideOpts = {
      duration: k.durations.SLIDE,
      timing: 'ease-in'
    };
    var cb = completedTurn;
    if (callback && callback != completedTurn) {
      cb = function () { callback(); completedTurn(); }
    }
    setX(upperPage(), 0, slideOpts, cb);
  }


  function slideOut(callback) {
    var slideOpts = {
      duration: k.durations.SLIDE,
      timing: 'ease-in'
    };
    var cb = completedTurn;
    if (callback && callback != completedTurn) {
      cb = function () { callback(); completedTurn(); }
    }
    setX(upperPage(), 0 - p.reader.properties.pageWidth, slideOpts, cb);
  }


  function slideToCursor(cursorX, callback) {
    setX(
      upperPage(),
      Math.min(0, cursorX - p.reader.properties.pageWidth),
      { duration: k.durations.FOLLOW_CURSOR },
      callback
    );
  }

  API.pageCount = p.pageCount;
  API.addPage = addPage;
  API.visiblePages = visiblePages;
  API.getPlace = getPlace;
  API.moveTo = moveTo;
  API.listenForInteraction = listenForInteraction;
  API.lift = lift;
  API.drop = drop;
  API.turning = turning;

  initialize();

  return API;
}


Monocle.pieceLoaded('flippers/slider');
Monocle.Flippers.Legacy = function (reader, setPageFn) {
  if (Monocle.Flippers == this) {
    return new Monocle.Flippers.Legacy(reader, setPageFn);
  }

  var k = {
    LEGACY_MESSAGE: "Your browser doesn't support Monocle's full feature set. " +
      'You could try <a href="http://mozilla.com/firefox">Firefox</a>, ' +
      'Apple\'s <a href="http://apple.com/safari">Safari</a> or ' +
      'Google\'s <a href="http://google.com/chrome">Chrome</a>.',
    buttonText: {
      PREV: "... Previous part",
      NEXT: "Next part..."
    }
  }

  var p = {
    pageCount: 1,
    divs: {}
  }

  var API = {
    constructor: Monocle.Flippers.Legacy,
    properties: p,
    constants: k
  }


  function initialize() {
    p.reader = reader;
    p.setPageFn = setPageFn;
  }


  function addPage(pageDiv) {
    p.page = pageDiv;
  }


  function visiblePages() {
    return [p.page];
  }


  function getPlace() {
    return p.reader.getBook().placeFor(p.page.contentDiv);
  }


  function moveTo(locus) {
    var rslt = p.setPageFn(p.page, locus, updateButtons);
    p.reader.dispatchEvent('monocle:turn');
    return rslt;
  }



  function overrideDimensions() {
    p.page.scrollerDiv.style.right = "0";
    p.page.scrollerDiv.style.overflow = "auto";
    p.page.contentDiv.style.position = "relative";
    p.page.contentDiv.style.width = "100%";
    p.page.contentDiv.style.minWidth = "0%";

    if (!p.divs.legacyMessage) {
      p.divs.legacyMessage = document.createElement('div');
      p.divs.legacyMessage.innerHTML = k.LEGACY_MESSAGE;
      p.divs.legacyMessage.style.cssText = Monocle.Styles.ruleText(
        Monocle.Styles.Flippers.Legacy.message
      );
      p.page.scrollerDiv.insertBefore(p.divs.legacyMessage, p.page.contentDiv);
    }

    if (!p.divs.prevButton) {
      p.divs.prevButton = document.createElement('div');
      p.divs.prevButton.innerHTML = k.buttonText.PREV;
      p.divs.prevButton.style.cssText = Monocle.Styles.ruleText(
        Monocle.Styles.Flippers.Legacy.button
      );
      p.page.scrollerDiv.insertBefore(p.divs.prevButton, p.page.contentDiv);
    }

    if (!p.divs.nextButton) {
      p.divs.nextButton = document.createElement('div');
      p.divs.nextButton.innerHTML = k.buttonText.NEXT;
      p.divs.nextButton.style.cssText = Monocle.Styles.ruleText(
        Monocle.Styles.Flippers.Legacy.button
      );
      p.page.scrollerDiv.appendChild(p.divs.nextButton);
    }
  }


  function updateButtons() {
    var cIndex = getPlace().properties.component.properties.index;
    if (cIndex == 0) {
      p.divs.legacyMessage.style.display = "block";
      p.divs.prevButton.style.display = "none";
    } else {
      p.divs.legacyMessage.style.display = "none";
      p.divs.prevButton.style.display = "block";
    }

    if (cIndex == p.reader.getBook().properties.lastCIndex) {
      p.divs.nextButton.style.display = "none";
    } else {
      p.divs.nextButton.style.display = "block";
    }
  }


  function listenForInteraction() {
    Monocle.addListener(
      p.divs.prevButton,
      'click',
      function () { moveTo({ percent: -0.5 }) }
    )
    Monocle.addListener(
      p.divs.nextButton,
      'click',
      function () { moveTo({ percent: 1.5 }) }
    )
  }


  API.pageCount = p.pageCount;
  API.addPage = addPage;
  API.visiblePages = visiblePages;
  API.getPlace = getPlace;
  API.moveTo = moveTo;
  API.listenForInteraction = listenForInteraction;
  API.overrideDimensions = overrideDimensions;

  initialize();

  return API;
}


Monocle.Styles.Flippers.Legacy = {
  message: {
    "border": "1px solid #987",
    "background": "#FFC",
    "color": "#333",
    "font-family": "Helvetica, Arial, sans-serif",
    "font-size": "11px",
    "margin-right": "3px",
    "margin-bottom": "3px",
    "padding": "0.5em 1em"
  },
  button: {
    "background": "#DDD",
    "padding": "6px",
    "border": "1px solid #666",
    "color": "#009",
    "font": "bold 12px Helvetica, Arial, sans-serif",
    "text-shadow": "-1px -1px #EEE",
    "border-radius": "5px",
    "margin-right": "3px",
    "margin-bottom": "3px",
    "margin-top": "3px"
  }
}


Monocle.pieceLoaded('flippers/legacy');
Monocle.Controls.Spinner = function (reader) {
  if (Monocle.Controls == this) {
    return new Monocle.Controls.Spinner(reader);
  }

  var k = {
    imgURI: "data:image/gif;base64,R0lGODlhGAAYALMAAAAAAJmZmUpKStra2jo6OjM" +
      "zM319fczMzLa2tuvr61FRUSMjI42NjV5eXkJCQv%2F%2F%2FyH5BAEHAA8ALAAAAAAY" +
      "ABgAAAS%2F8MlJq73zGEVKIQozYNRgdF6aGgmJOF54SAMjeM5sHZ2AYIGebjJQFBStz" +
      "PAxaBQcSYlTMKIAAJVBx0D0%2FCpXC8JTNRQEl7DlxpDcAmmsxayQkOOXw9NewFyXEg" +
      "kefBgLAAtRgR8SHYASB1cEiWMOEkZtFwFXDRR0EgxHJAxXmAlsNB1fFwoAnA8BT1UPZ" +
      "lAYCQcjPAWYNDBUJA8HN0gVugJwmTA5eUZnBjoaMEeORCcqKgQMiRg1HB4g2sDiwBEA" +
      "Ow%3D%3D"
  }

  var p = {
    divs: []
  }

  var API = {
    constructor: Monocle.Controls.Spinner,
    constants: k,
    properties: p
  }


  function initialize() {
    p.reader = reader;
  }


  function createControlElements() {
    var anim = document.createElement('div');
    anim.style.cssText = Monocle.Styles.ruleText(
      Monocle.Styles.Controls.Spinner.anim
    );
    anim.style.backgroundImage = "url(" + k.imgURI + ")";
    p.divs.push(anim);
    return anim;
  }


  function listenForUsualDelays() {
    p.reader.addListener('monocle:bookchanging', spin);
    p.reader.addListener('monocle:bookchange', spun);
    p.reader.addListener('monocle:resizing', spin);
    p.reader.addListener('monocle:resize', spun);
  }


  function spin() {
    p.reader.showControl(API);
  }


  function spun() {
    p.reader.hideControl(API);
  }

  API.createControlElements = createControlElements;
  API.listenForUsualDelays = listenForUsualDelays;
  API.spin = spin;
  API.spun = spun;

  initialize();

  return API;
}



Monocle.Styles.Controls.Spinner = {
  anim: {
    "position": "absolute",
    "width": "100%",
    "height": "100%",
    "background-color": "#FFF",
    "background-repeat": "no-repeat",
    "background-position": "center center"
  }
}


Monocle.pieceLoaded('controls/spinner');
Monocle.Controls.Magnifier = function (reader) {
  if (Monocle.Controls == this) {
    return new Monocle.Controls.Magnifier(reader);
  }

  var k = {
    LARGE_FONT_SIZE: "115%",
    NORMAL_FONT_SIZE: Monocle.Styles.content['font-size'] || "100%"
  }

  var p = {
    buttons: [],
    enlarged: false
  }

  var API = {
    constructor: Monocle.Controls.Magnifier,
    constants: k,
    properties: p
  }


  function initialize() {
    p.reader = reader;
  }


  function createControlElements() {
    var btn = document.createElement('div');
    btn.style.cssText = Monocle.Styles.ruleText(
      Monocle.Styles.Controls.Magnifier.button
    );
    btn.smallA = document.createElement('span');
    btn.smallA.style.cssText = Monocle.Styles.ruleText(
      Monocle.Styles.Controls.Magnifier.smallA
    )
    btn.appendChild(btn.smallA);
    btn.largeA = document.createElement('span');
    btn.largeA.style.cssText = Monocle.Styles.ruleText(
      Monocle.Styles.Controls.Magnifier.largeA
    )
    btn.largeA.innerHTML = btn.smallA.innerHTML = "A";
    btn.appendChild(btn.largeA);

    var evtType = "touchstart";
    Monocle.addListener(btn, evtType, toggleMagnification, true);

    p.buttons.push(btn);
    return btn;
  }


  function toggleMagnification(evt) {
    if (evt.preventDefault) {
      evt.preventDefault();
      evt.stopPropagation();
    } else {
      evt.returnValue = false;
    }
    var opacities;
    if (!p.enlarged) {
      Monocle.Styles.content['font-size'] = k.LARGE_FONT_SIZE;
      opacities = [0.3, 1]
      p.enlarged = true;
    } else {
      Monocle.Styles.content['font-size'] = k.NORMAL_FONT_SIZE;
      opacities = [1, 0.3]
      p.enlarged = false;
    }

    for (var i = 0; i < p.buttons.length; i++) {
      p.buttons[i].smallA.style.opacity = opacities[0];
      p.buttons[i].largeA.style.opacity = opacities[1];
    }

    p.reader.reapplyStyles();
  }

  API.createControlElements = createControlElements;

  initialize();

  return API;
}


Monocle.Styles.Controls.Magnifier = {
  button: {
    "cursor": "pointer",
    "color": "#555",
    "position": "absolute",
    "top": "2px",
    "right": "10px",
    "padding": "0 2px"
  },
  largeA: {
    "font-size": "18px",
    "opacity": "0.3"
  },
  smallA: {
    "font-size": "11px"
  }
}

Monocle.pieceLoaded('controls/magnifier');
Monocle.Controls.Scrubber = function (reader) {
  if (Monocle.Controls == this) {
    return new Monocle.Controls.Scrubber(reader);
  }

  var k = {
  }

  var p = {
    divs: {}
  }

  var API = {
    constructor: Monocle.Controls.Scrubber,
    constants: k,
    properties: p
  }


  function initialize() {
    p.reader = reader;
    p.book = p.reader.getBook();
    p.reader.addListener('monocle:turn', updateNeedles);
    updateNeedles();
  }


  function calcLeftBound(cntr) {
    if (p.rightBound == cntr.offsetWidth) {
      return;
    }
    p.rightBound = cntr.offsetWidth;
    p.leftBound = p.reader.properties.boxDimensions.left;
    var box = cntr;
    while (box && box != p.reader.properties.divs.box) {
      p.leftBound += box.offsetLeft;
      box = box.parentNode;
    }
  }


  function rebaseX(evt, cntr) {
    calcLeftBound(cntr);
    var x = evt.pageX;
    if (evt.changedTouches) {
      x = evt.changedTouches[0].pageX;
    }
    return Math.max(Math.min(p.rightBound, x - p.leftBound), 0);
  }


  function pixelToPlace(x, cntr) {
    if (!p.componentIds) {
      p.componentIds = p.reader.getBook().properties.componentIds;
      p.componentWidth = 100 / p.componentIds.length;
    }
    var pc = (x / cntr.offsetWidth) * 100;
    var cmpt = p.componentIds[Math.floor(pc / p.componentWidth)];
    var cmptPc = ((pc % p.componentWidth) / p.componentWidth);
    return { componentId: cmpt, percentageThrough: cmptPc };
  }


  function placeToPixel(place, cntr) {
    if (!p.componentIds) {
      p.componentIds = p.reader.getBook().properties.componentIds;
      p.componentWidth = 100 / p.componentIds.length;
    }
    var componentIndex = p.componentIds.indexOf(place.componentId());
    var pc = p.componentWidth * componentIndex;
    pc += place.percentageThrough() * p.componentWidth;
    return Math.round((pc / 100) * cntr.offsetWidth);
  }


  function updateNeedles() {
    if (p.hidden || !p.divs.container) {
      return;
    }
    var place = p.reader.getPlace();
    var x = placeToPixel(place, p.divs.container[0]);
    for (var i = 0; i < p.divs.needle.length; ++i) {
      setX(p.divs.needle[i], x - p.divs.needle[i].offsetWidth / 2);
    }
  }


  function eventType(str) {
    var evtTypeMap;
   
      evtTypeMap = {
        "start": "touchstart",
        "move": "touchmove",
        "end": "touchend"
      }
    return evtTypeMap[str];
  }


  function createDivNamed(name, parentNode) {
    var div = document.createElement('div');
    div.style.cssText = Monocle.Styles.ruleText(
      Monocle.Styles.Controls.Scrubber[name]
    );
    if (parentNode) {
      parentNode.appendChild(div)
    }
    p.divs[name] = p.divs[name] || [];
    p.divs[name].push(div);

    return div;
  }


  function setX(node, x) {
    x = Math.min(p.divs.container[0].offsetWidth - node.offsetWidth, x);
    x = Math.max(x, 0);
    node.style.webkitTransform =
      node.style.MozTransform =
        "translateX(" + x + "px)";
  }


  function createControlElements() {
    var cntr = createDivNamed('container');
    var track = createDivNamed('track', cntr);
    var needle = createDivNamed('needle', cntr);
    var bubble = createDivNamed('bubble', cntr);

    var moveEvt = function (evt, x) {
      evt.stopPropagation();
      evt.preventDefault();
      x = x || rebaseX(evt, cntr);
      var place = pixelToPlace(x, cntr);
      setX(needle, x - needle.offsetWidth / 2);
      var chps = p.book.chaptersForComponent(place.componentId);
      var cmptIndex = p.componentIds.indexOf(place.componentId);
      var chp = chps[Math.floor(chps.length * place.percentageThrough)];
      if (cmptIndex > -1 && p.book.properties.components[cmptIndex]) {
        var actualPlace = Monocle.Place.FromPercentageThrough(
          p.book.properties.components[cmptIndex],
          place.percentageThrough
        );
        chp = actualPlace.chapterInfo() || chp;
      }

      if (chp) {
        bubble.innerHTML = chp.title;
      }
      setX(bubble, x - bubble.offsetWidth / 2);

      p.lastX = x;
      return place;
    }

    var endEvt = function (evt) {
      var place = moveEvt(evt, p.lastX);
      p.reader.moveTo({
        percent: place.percentageThrough,
        componentId: place.componentId
      });
      Monocle.removeListener(cntr, eventType('move'), moveEvt);
      Monocle.removeListener(document.body, eventType('end'), endEvt);
      bubble.style.display = "none";
    }

    Monocle.addListener(
      cntr,
      eventType("start"),
      function (evt) {
        bubble.style.display = "block";
        moveEvt(evt);
        Monocle.addListener(cntr, eventType('move'), moveEvt);
        Monocle.addListener(document.body, eventType("end"), endEvt);
      }
    );

    return cntr;
  }

  API.createControlElements = createControlElements;
  API.updateNeedles = updateNeedles;

  initialize();

  return API;
}


Monocle.Styles.Controls.Scrubber = {
  container: {
    "position": "absolute",
    "left": "1em",
    "right": "1em",
    "bottom": "8px",
    "height": "30px",
    "background": "#FFF",
    "-webkit-transform-style": "preserve-3d"
  },
  track: {
    "margin-top": "13px",
    "border-top": "2px groove #CCC",
    "-webkit-transform-style": "preserve-3d"
  },
  needle: {
    "position": "absolute",
    "width": "1px",
    "height": "12px",
    "top": "7px",
    "border": "2px solid #777",
    "-webkit-transform-style": "preserve-3d"
  },
  bubble: {
    "position": "absolute",
    "bottom": "2.5em",
    "background": "rgba(0, 0, 0, 0.9)",
    "-webkit-box-shadow": "2px 2px 8px #000",
    "-webkit-border-radius": "10px",
    "-moz-border-radius": "10px",
    "padding": "12px",
    "display": "none",
    "font": "bold 12px Lucida Grande, Helvetica, sans-serif",
    "color": "#CCC",
    "text-shadow": "1px 1px #333",
    "white-space": "nowrap",
    "min-width": "8em",
    "max-width": "12em",
    "text-overflow": "ellipsis",
    "overflow": "hidden",
    "-webkit-transform-style": "preserve-3d"
  }
}


Monocle.pieceLoaded('controls/scrubber');
Monocle.Controls.PlaceSaver = function (reader) {
  if (Monocle.Controls == this) {
    return new Monocle.Controls.PlaceSaver(reader);
  }

  var k = {
    COOKIE_NAMESPACE: "monocle.controls.placesaver.",
    COOKIE_EXPIRES_IN_DAYS: 7 
  }

  var p = {
  }

  var API = {
    constructor: Monocle.Controls.PlaceSaver,
    constants: k,
    properties: p
  }


  function initialize() {
    p.reader = reader;
    applyToBook(p.reader.getBook());
    p.reader.addListener('monocle:turn', savePlaceToCookie);
    p.reader.addListener('monocle:bookChange', applyToBook);
  }


  function applyToBook() {
    p.bkTitle = p.reader.getBook().getMetaData('title');
    p.bkTitle = p.bkTitle.toLowerCase().replace(/[^a-z0-9]/g, '');
    p.prefix = k.COOKIE_NAMESPACE +
      p.reader.properties.divs.box.id + "." +
      p.bkTitle + ".";
  }


  function setCookie(key, value, days) {
    var expires = "";
    if (days) {
      var d = new Date();
      d.setTime(d.getTime() + (days * 24 * 60 * 60 * 1000));
      expires = "; expires="+d.toGMTString();
    }
    var path = "; path=/";
    document.cookie = p.prefix + key + " = " + value + expires + path;
    return value;
  }


  function getCookie(key) {
    if (!document.cookie) {
      return null;
    }
    var regex = new RegExp(p.prefix + key + "=(.+?)(;|$)");
    var matches = document.cookie.match(regex);
    if (matches) {
      return matches[1];
    } else {
      return null;
    }
  }


  function savePlaceToCookie() {
    var place = p.reader.getPlace();
    setCookie(
      "component",
      encodeURIComponent(place.componentId()),
      k.COOKIE_EXPIRES_IN_DAYS
    );
    setCookie(
      "percent",
      place.percentageThrough(),
      k.COOKIE_EXPIRES_IN_DAYS
    );
  }


  function savedPlace() {
    var place = {
      component: getCookie('component'),
      percent: getCookie('percent')
    }
    if (place.component && place.percent) {
      place.component = decodeURIComponent(place.component);
      place.percent = parseFloat(place.percent);
      return place;
    } else {
      return null;
    }
  }


  function restorePlace() {
    var place = savedPlace();
    if (place) {
      p.reader.moveTo({ percent: place.percent, componentId: place.component });
    }
  }


  API.savedPlace = savedPlace;
  API.restorePlace = restorePlace;

  initialize();

  return API;
}


Monocle.pieceLoaded('controls/placesaver');
Monocle.Controls.Contents = function (reader) {
  if (Monocle.Controls == this) {
    return new Monocle.Controls.Contents(reader);
  }

  var k = {
    CHAPTER_INVOKE_EVENT: 'click'
  }

  var p = {
    divs: []
  }

  var API = {
    constructor: Monocle.Controls.Contents,
    constants: k,
    properties: p
  }


  function initialize() {
    p.reader = reader;
  }


  function createControlElements() {
    var div = document.createElement('div');
    p.divs.push(div);
    div.style.cssText = Monocle.Styles.ruleText(
      Monocle.Styles.Controls.Contents.container
    );
    contentsForBook(div, reader.getBook());
    return div;
  }


  function contentsForBook(div, book) {
    while (div.hasChildNodes()) {
      div.removeChild(div.firstChild);
    }
    var list = document.createElement('ol');
    div.appendChild(list);
    list.style.cssText = Monocle.Styles.ruleText(
      Monocle.Styles.Controls.Contents.list
    );

    var contents = book.chapterTree();
    for (var i = 0; i < contents.length; ++i) {
      chapterBuilder(list, contents[i], 0);
    }
  }


  function chapterBuilder(list, chp, padLvl) {
    var li = document.createElement('li');
    var span = document.createElement('span');
    span.style.paddingLeft = padLvl + "em";
    span.innerHTML = chp.title;
    li.appendChild(span);
    Monocle.addListener(
      li,
      k.CHAPTER_INVOKE_EVENT,
      function () {
        p.reader.skipToChapter(chp.src);
        p.reader.hideControl(API);
      }
    );
    li.style.cssText = Monocle.Styles.ruleText(
      Monocle.Styles.Controls.Contents.chapter
    );
    list.appendChild(li);
    if (chp.children) {
      for (var i = 0; i < chp.children.length; ++i) {
        chapterBuilder(list, chp.children[i], padLvl + 1);
      }
    }
  }


  API.createControlElements = createControlElements;

  initialize();

  return API;
}


Monocle.Styles.Controls.Contents = {
  container: {
    "position": "absolute",
    "width": "75%",
    "height": "75%",
    "left": "12.5%",
    "top": "12.5%",
    "background": "#EEE",
    "border": "2px solid #F7F7F7",
    "overflow-y": "auto",
    "border-radius": "9px",
    "-moz-border-radius": "9px",
    "-webkit-border-radius": "9px",
    "box-shadow": "1px 2px 6px rgba(0,0,0,0.5)",
    "-moz-box-shadow": "1px 2px 6px rgba(0,0,0,0.5)",
    "-webkit-box-shadow": "1px 2px 6px rgba(0,0,0,0.5)"
  },
  list: {
    "margin": "6px",
    "padding": "0"
  },
  chapter: {
    "list-style": "none",
    "line-height": "220%",
    "padding-left": "1em",
    "padding-right": "2em",
    "border-bottom": "2px groove #FEFEFE",
    "cursor": "pointer"
  }
}

Monocle.pieceLoaded('controls/contents');
